Class: Conjoin::Widgets::Base

Inherits:
Object
  • Object
show all
Defined in:
lib/conjoin/widgets.rb

Constant Summary collapse

JS_ESCAPE =
{ '\\' => '\\\\', '</' => '<\/', "\r\n" => '\n', "\n" => '\n', "\r" => '\n', '"' => '\\"', "'" => "\\'" }

Class Attribute Summary collapse

Instance Attribute Summary collapse

Class Method Summary collapse

Instance Method Summary collapse

Constructor Details

#initialize(app, res, req, settings, event, folder, options) ⇒ Base

Returns a new instance of Base.



137
138
139
140
141
142
143
144
145
146
147
148
149
150
151
152
153
# File 'lib/conjoin/widgets.rb', line 137

def initialize app, res, req, settings, event, folder, options
  @app      = app
  @res      = res
  @req      = req
  @settings = settings
  @event    = event
  @folder   = folder
  @options  = options

  # add the widget to the req widgets
  req.env[:widgets] ||= {}
  unless req.env[:widgets][folder]
    req.env[:widgets][folder] = {}
  end

  event.add_observer self, :trigger_event
end

Class Attribute Details

.available_helper_methods ⇒ Object

Returns the value of attribute available_helper_methods.



276
277
278
# File 'lib/conjoin/widgets.rb', line 276

def available_helper_methods
  @available_helper_methods
end

.events ⇒ Object

Returns the value of attribute events.



276
277
278
# File 'lib/conjoin/widgets.rb', line 276

def events
  @events
end

Instance Attribute Details

#app ⇒ Object

Returns the value of attribute app.



135
136
137
# File 'lib/conjoin/widgets.rb', line 135

def app
  @app
end

#event ⇒ Object

Returns the value of attribute event.



135
136
137
# File 'lib/conjoin/widgets.rb', line 135

def event
  @event
end

#folder ⇒ Object

Returns the value of attribute folder.



135
136
137
# File 'lib/conjoin/widgets.rb', line 135

def folder
  @folder
end

#options ⇒ Object

Returns the value of attribute options.



135
136
137
# File 'lib/conjoin/widgets.rb', line 135

def options
  @options
end

#req ⇒ Object

Returns the value of attribute req.



135
136
137
# File 'lib/conjoin/widgets.rb', line 135

def req
  @req
end

#res ⇒ Object

Returns the value of attribute res.



135
136
137
# File 'lib/conjoin/widgets.rb', line 135

def res
  @res
end

#settings ⇒ Object

Returns the value of attribute settings.



135
136
137
# File 'lib/conjoin/widgets.rb', line 135

def settings
  @settings
end

Class Method Details

.helper_method(method) ⇒ Object



290
291
292
293
# File 'lib/conjoin/widgets.rb', line 290

def helper_method method
  @available_helper_methods ||= []
  @available_helper_methods << method
end

.helper_methods(*methods) ⇒ Object



295
296
297
298
299
# File 'lib/conjoin/widgets.rb', line 295

def helper_methods *methods
  methods.each do |method|
    helper_method method
  end
end

.respond_to(event, opts = {}) ⇒ Object



278
279
280
281
# File 'lib/conjoin/widgets.rb', line 278

def respond_to event, opts = {}
  @events ||= []
  @events << [event, opts]
end

.responds_to(*events) ⇒ Object



283
284
285
286
287
288
# File 'lib/conjoin/widgets.rb', line 283

def responds_to *events
  @events ||= []
  events.each do |event|
    @events << [event, {}]
  end
end

Instance Method Details

#add_after(selector, opts = {}) ⇒ Object



197
198
199
200
# File 'lib/conjoin/widgets.rb', line 197

def add_after selector, opts = {}
  content = render opts
  res.write '$("' + selector + '").after("'+ escape(content) +'");'
end

#append(selector, opts = {}) ⇒ Object



192
193
194
195
# File 'lib/conjoin/widgets.rb', line 192

def append selector, opts = {}
  content = render opts
  res.write '$("' + selector + '").append("'+ escape(content) +'");'
end

#attrs_for(selector, opts = {}) ⇒ Object



202
203
204
# File 'lib/conjoin/widgets.rb', line 202

def attrs_for selector, opts = {}
  res.write '$("' + selector + '").attr('+ (opts.to_json) +');'
end

#current_user ⇒ Object



155
156
157
# File 'lib/conjoin/widgets.rb', line 155

def current_user
  app.current_user
end

#escape(js) ⇒ Object



206
207
208
# File 'lib/conjoin/widgets.rb', line 206

def escape js
  js.to_s.gsub(/(\\|<\/|\r\n|\\3342\\2200\\2250|[\n\r"'])/) {|match| JS_ESCAPE[match] }
end

#hide(selector) ⇒ Object



188
189
190
# File 'lib/conjoin/widgets.rb', line 188

def hide selector
  res.write '$("' + selector + '").hide();'
end

#id_for(state) ⇒ Object



159
160
161
# File 'lib/conjoin/widgets.rb', line 159

def id_for state
  "#{req.env[:widget_name]}_#{state}"
end

#page_change ⇒ Object



163
164
165
166
# File 'lib/conjoin/widgets.rb', line 163

def page_change
  res.headers["Content-Type"] = "text/javascript; charset=utf-8"
  res.write '$(document).trigger("page:change");'
end

#render(*args) ⇒ Object



226
227
228
229
230
231
232
233
234
235
236
237
238
239
240
241
242
243
244
245
246
247
248
249
250
251
252
253
254
255
256
257
258
259
260
261
262
263
264
265
266
267
# File 'lib/conjoin/widgets.rb', line 226

def render *args
  if args.first.kind_of? Hash
    locals = args.first
    # if it's a partial we add an underscore infront of it
    state = view = locals[:state] || "#{locals[:partial]}".gsub(/([a-zA-Z_]+)$/, '_\1')
  else
    state = view = args.first
    locals = args.length > 1 ? args.last : {}
  end

  unless view
    state = view = caller[0][/`.*'/][1..-2]

    if (options.key?(:from_event) and !options.key?(:replace))
      @options[:cache] = false
    end
  end

  if locals.key?(:state) and state.to_s == view
    return send state
  end

  tmpl_engine = settings[:render][:template_engine]

  if (req_helper_methods = req.env[:widgets][folder][:req_helper_methods]) \
  and (!options.key?(:cache))
    locals.merge! req_helper_methods
  else
    req.env[:widgets][folder][:req_helper_methods] = {}

    helper_methods.each do |method|
      req.env[:widgets][folder][:req_helper_methods][method] = locals[method] = self.send method
    end
  end

  req.env[:widget_name]  = folder
  req.env[:widget_state] = state

  locals[:w] = locals[:widget] = self

  app.render "#{app.widgets_root}/#{folder}/#{view}.#{tmpl_engine}", locals
end

#replace(state, opts = {}) ⇒ Object



168
169
170
171
172
173
174
175
176
177
178
179
180
181
182
183
184
185
186
# File 'lib/conjoin/widgets.rb', line 168

def replace state, opts = {}
  @options[:replace] = true

  if !state.is_a? String
    content = render state, opts
    selector = '#' + id_for(state)
  else
    if !opts.key?(:content) and !opts.key?(:with)
      content = render opts
    else
      content = opts[:content] || opts[:with]
    end
    selector = state
  end

  res.write '$("' + selector + '").replaceWith("' + escape(content) + '");'
  # scroll to the top of the page just as if we went to the url directly
  res.write 'window.scrollTo(0, 0);'
end

#trigger_event(event, widget_name, data = {}) ⇒ Object



210
211
212
213
214
215
216
217
218
219
220
221
222
223
224
# File 'lib/conjoin/widgets.rb', line 210

def trigger_event event, widget_name, data = {}
  if events = self.class.events
    events.each do |class_event, opts|
      if class_event == event && (widget_name == folder.to_s or opts[:for].to_s == widget_name)
        event = opts[:with] if opts[:with]

        if method(event).parameters.length > 0
          send(event, data)
        else
          send(event)
        end
      end
    end
  end
end